package kr.co.vividnext.sodalive.can

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.can.QCan.can1
import kr.co.vividnext.sodalive.can.charge.Charge
import kr.co.vividnext.sodalive.can.charge.ChargeStatus
import kr.co.vividnext.sodalive.can.charge.QCharge.charge
import kr.co.vividnext.sodalive.can.payment.PaymentGateway
import kr.co.vividnext.sodalive.can.payment.PaymentStatus
import kr.co.vividnext.sodalive.can.payment.QPayment.payment
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.can.use.QUseCan.useCan
import kr.co.vividnext.sodalive.can.use.UseCan
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.QMember
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.domain.Pageable
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface CanRepository : JpaRepository<Can, Long>, CanQueryRepository

interface CanQueryRepository {
    fun findAllByStatus(status: CanStatus): List<CanResponse>
    fun getCanUseStatus(member: Member, pageable: Pageable): List<UseCan>
    fun getCanChargeStatus(member: Member, pageable: Pageable, container: String): List<Charge>
    fun isExistPaidLiveRoom(memberId: Long, roomId: Long): UseCan?
    fun getCanUsedForLiveRoomNotRefund(memberId: Long, roomId: Long, canUsage: CanUsage = CanUsage.LIVE): UseCan?
}

@Repository
class CanQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : CanQueryRepository {
    override fun findAllByStatus(status: CanStatus): List<CanResponse> {
        return queryFactory
            .select(
                QCanResponse(
                    can1.id,
                    can1.title,
                    can1.can,
                    can1.rewardCan,
                    can1.price
                )
            )
            .from(can1)
            .where(can1.status.eq(status))
            .orderBy(can1.can.asc())
            .fetch()
    }

    override fun getCanUseStatus(member: Member, pageable: Pageable): List<UseCan> {
        return queryFactory
            .selectFrom(useCan)
            .where(useCan.member.id.eq(member.id))
            .offset(pageable.offset)
            .limit(pageable.pageSize.toLong())
            .orderBy(useCan.id.desc())
            .fetch()
    }

    override fun getCanChargeStatus(member: Member, pageable: Pageable, container: String): List<Charge> {
        val qMember = QMember.member
        val chargeStatusCondition = when (container) {
            "aos" -> {
                charge.payment.paymentGateway.eq(PaymentGateway.PG)
                    .or(charge.payment.paymentGateway.eq(PaymentGateway.GOOGLE_IAP))
            }

            "ios" -> {
                charge.payment.paymentGateway.eq(PaymentGateway.PG)
                    .or(charge.payment.paymentGateway.eq(PaymentGateway.APPLE_IAP))
            }

            else -> charge.payment.paymentGateway.eq(PaymentGateway.PG)
        }

        return queryFactory
            .selectFrom(charge)
            .innerJoin(charge.member, qMember)
            .leftJoin(charge.useCan, useCan)
            .leftJoin(charge.payment, payment)
            .where(
                qMember.id.eq(member.id)
                    .and(
                        payment.status.eq(PaymentStatus.COMPLETE)
                            .or(
                                charge.status.eq(ChargeStatus.REFUND_CHARGE)
                                    .and(useCan.isNotNull)
                            )
                            .or(charge.status.eq(ChargeStatus.EVENT))
                            .or(charge.status.eq(ChargeStatus.ADMIN))
                    )
                    .and(chargeStatusCondition)
            )
            .offset(pageable.offset)
            .limit(pageable.pageSize.toLong())
            .orderBy(charge.id.desc())
            .fetch()
    }

    override fun isExistPaidLiveRoom(memberId: Long, roomId: Long): UseCan? {
        return queryFactory
            .selectFrom(useCan)
            .innerJoin(useCan.member, member)
            .innerJoin(useCan.room, liveRoom)
            .where(
                member.id.eq(memberId)
                    .and(liveRoom.id.eq(roomId))
                    .and(useCan.canUsage.eq(CanUsage.LIVE))
            )
            .orderBy(useCan.id.desc())
            .fetchFirst()
    }

    override fun getCanUsedForLiveRoomNotRefund(memberId: Long, roomId: Long, canUsage: CanUsage): UseCan? {
        return queryFactory
            .selectFrom(useCan)
            .innerJoin(useCan.member, member)
            .innerJoin(useCan.room, liveRoom)
            .where(
                member.id.eq(memberId)
                    .and(liveRoom.id.eq(roomId))
                    .and(useCan.canUsage.eq(canUsage))
                    .and(useCan.isRefund.isFalse)
            )
            .orderBy(useCan.id.desc())
            .fetchFirst()
    }
}
