package kr.co.vividnext.sodalive.content.main

import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBannerType
import kr.co.vividnext.sodalive.content.main.banner.GetAudioContentBannerResponse
import kr.co.vividnext.sodalive.content.main.curation.GetAudioContentCurationResponse
import kr.co.vividnext.sodalive.content.order.OrderService
import kr.co.vividnext.sodalive.content.theme.AudioContentThemeQueryRepository
import kr.co.vividnext.sodalive.event.EventItem
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.block.BlockMemberRepository
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Service

@Service
class AudioContentMainService(
    private val repository: AudioContentRepository,
    private val blockMemberRepository: BlockMemberRepository,
    private val orderService: OrderService,
    private val audioContentThemeRepository: AudioContentThemeQueryRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) {
    fun getMain(member: Member): GetAudioContentMainResponse {
        val isAdult = member.auth != null

        // 2주일 이내에 콘텐츠를 올린 크리에이터 20명 조회
        val newContentUploadCreatorList = repository.getNewContentUploadCreatorList(
            cloudfrontHost = imageHost,
            isAdult = isAdult
        )
            .asSequence()
            .filter { !blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it.creatorId) }
            .toList()

        val bannerList = repository.getAudioContentMainBannerList(isAdult = isAdult)
            .asSequence()
            .filter {
                if (it.type == AudioContentBannerType.CREATOR && it.creator != null) {
                    !blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it.creator!!.id!!)
                } else {
                    true
                }
            }
            .map {
                GetAudioContentBannerResponse(
                    type = it.type,
                    thumbnailImageUrl = "$imageHost/${it.thumbnailImage}",
                    eventItem = if (it.type == AudioContentBannerType.EVENT && it.event != null) {
                        EventItem(
                            id = it.event!!.id!!,
                            thumbnailImageUrl = if (!it.event!!.thumbnailImage.startsWith("https://")) {
                                "$imageHost/${it.event!!.thumbnailImage}"
                            } else {
                                it.event!!.thumbnailImage
                            },
                            detailImageUrl = if (
                                it.event!!.detailImage != null &&
                                !it.event!!.detailImage!!.startsWith("https://")
                            ) {
                                "$imageHost/${it.event!!.detailImage}"
                            } else {
                                it.event!!.detailImage
                            },
                            popupImageUrl = null,
                            link = it.event!!.link,
                            title = it.event!!.title,
                            isPopup = false
                        )
                    } else {
                        null
                    },
                    creatorId = if (it.type == AudioContentBannerType.CREATOR && it.creator != null) {
                        it.creator!!.id
                    } else {
                        null
                    },
                    link = it.link
                )
            }
            .toList()

        // 구매목록 20개
        val orderList = orderService.getAudioContentMainOrderList(
            member = member,
            limit = 20
        )

        // 콘텐츠 테마
        val themeList = audioContentThemeRepository.getActiveThemeOfContent(isAdult = isAdult)

        // 새 콘텐츠 20개 - 시간 내림차순 정렬
        val newContentList = repository.findByTheme(
            cloudfrontHost = imageHost,
            isAdult = isAdult
        )
            .asSequence()
            .filter { !blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it.creatorId) }
            .toList()

        val curationList = repository
            .getAudioContentCurations(isAdult = isAdult)
            .asSequence()
            .map {
                GetAudioContentCurationResponse(
                    title = it.title,
                    description = it.description,
                    contents = repository.findAudioContentByCurationId(
                        curationId = it.id!!,
                        cloudfrontHost = imageHost,
                        isAdult = isAdult
                    )
                        .asSequence()
                        .filter { content ->
                            !blockMemberRepository.isBlocked(
                                blockedMemberId = member.id!!,
                                memberId = content.creatorId
                            )
                        }
                        .toList()
                )
            }
            .filter { it.contents.isNotEmpty() }
            .toList()

        return GetAudioContentMainResponse(
            newContentUploadCreatorList = newContentUploadCreatorList,
            bannerList = bannerList,
            orderList = orderList,
            themeList = themeList,
            newContentList = newContentList,
            curationList = curationList
        )
    }

    fun getNewContentByTheme(theme: String, member: Member): List<GetAudioContentMainItem> {
        return repository.findByTheme(
            cloudfrontHost = imageHost,
            theme = theme,
            isAdult = member.auth != null
        )
            .asSequence()
            .filter { !blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it.creatorId) }
            .toList()
    }
}
