package kr.co.vividnext.sodalive.live.reservation

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.live.reservation.QLiveReservation.liveReservation
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface LiveReservationRepository : JpaRepository<LiveReservation, Long>, LiveReservationQueryRepository

interface LiveReservationQueryRepository {
    fun getReservationList(roomId: Long): List<LiveReservation>

    fun cancelReservation(roomId: Long)

    fun getReservationBookerList(roomId: Long): List<Member>

    fun isExistsReservation(roomId: Long, memberId: Long): Boolean
    fun getReservationListByMemberId(memberId: Long, active: Boolean): List<LiveReservation>

    fun getReservationByReservationAndMemberId(
        reservationId: Long,
        memberId: Long
    ): LiveReservation?

    fun getReservationCount(memberId: Long): Int
}

@Repository
class LiveReservationQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : LiveReservationQueryRepository {
    override fun getReservationList(roomId: Long): List<LiveReservation> {
        return queryFactory
            .selectFrom(liveReservation)
            .innerJoin(liveReservation.room, liveRoom)
            .where(
                liveRoom.id.eq(roomId)
                    .and(liveReservation.isActive.isTrue)
            )
            .fetch()
    }

    override fun cancelReservation(roomId: Long) {
        queryFactory
            .update(liveReservation)
            .set(liveReservation.isActive, false)
            .where(liveReservation.room.id.eq(roomId))
            .execute()
    }

    override fun getReservationBookerList(roomId: Long): List<Member> {
        return queryFactory
            .select(member)
            .from(liveReservation)
            .innerJoin(liveReservation.member, member)
            .innerJoin(liveReservation.room, liveRoom)
            .where(
                liveRoom.id.eq(roomId)
                    .and(liveReservation.isActive.isTrue)
            )
            .fetch()
    }

    override fun isExistsReservation(roomId: Long, memberId: Long): Boolean {
        return queryFactory
            .selectFrom(liveReservation)
            .innerJoin(liveReservation.member, member)
            .innerJoin(liveReservation.room, liveRoom)
            .where(
                liveReservation.isActive.isTrue
                    .and(liveReservation.room.id.eq(roomId))
                    .and(liveReservation.member.id.eq(memberId))
            )
            .fetchFirst() != null
    }

    override fun getReservationListByMemberId(memberId: Long, active: Boolean): List<LiveReservation> {
        return queryFactory
            .selectFrom(liveReservation)
            .innerJoin(liveReservation.member, member)
            .innerJoin(liveReservation.room, liveRoom)
            .where(
                liveReservation.isActive.eq(active)
                    .and(member.id.eq(memberId))
            )
            .fetch()
    }

    override fun getReservationByReservationAndMemberId(reservationId: Long, memberId: Long): LiveReservation? {
        return queryFactory
            .selectFrom(liveReservation)
            .innerJoin(liveReservation.member, member)
            .innerJoin(liveReservation.room, liveRoom)
            .where(
                liveReservation.id.eq(reservationId)
                    .and(member.id.eq(memberId))
            )
            .fetchFirst()
    }

    override fun getReservationCount(memberId: Long): Int {
        return queryFactory
            .selectFrom(liveReservation)
            .where(
                liveReservation.member.id.eq(memberId)
                    .and(liveReservation.isActive.isTrue)
                    .and(liveReservation.room.isActive.isTrue)
            )
            .fetch()
            .size
    }
}
