package kr.co.vividnext.sodalive.live.tag

import com.amazonaws.services.s3.model.ObjectMetadata
import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile

@Service
class LiveTagService(
    private val repository: LiveTagRepository,

    private val objectMapper: ObjectMapper,
    private val s3Uploader: S3Uploader,

    @Value("\${cloud.aws.s3.bucket}")
    private val coverImageBucket: String,
    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) {
    @Transactional
    fun enrollmentLiveTag(image: MultipartFile, requestString: String) {
        val request = objectMapper.readValue(requestString, CreateLiveTagRequest::class.java)
        tagExistCheck(request)

        val tag = repository.save(LiveTag(request.tag, isAdult = request.isAdult))

        val metadata = ObjectMetadata()
        metadata.contentLength = image.size

        val tagImageFileName = generateFileName(prefix = "${tag.id}-")
        val tagImagePath = s3Uploader.upload(
            inputStream = image.inputStream,
            bucket = coverImageBucket,
            filePath = "live_cover/${tag.id}/$tagImageFileName",
            metadata = metadata
        )

        tag.image = tagImagePath
    }

    @Transactional
    fun deleteTag(id: Long) {
        val tag = repository.findByIdOrNull(id)
            ?: throw SodaException("잘못된 요청입니다.")

        tag.tag = "${tag.tag}_deleted"
        tag.isActive = false
    }

    @Transactional
    fun modifyTag(id: Long, image: MultipartFile?, requestString: String) {
        val tag = repository.findByIdOrNull(id)
            ?: throw SodaException("잘못된 요청입니다.")

        val request = objectMapper.readValue(requestString, CreateLiveTagRequest::class.java)
        tag.tag = request.tag
        tag.isAdult = request.isAdult

        if (image != null) {
            val metadata = ObjectMetadata()
            metadata.contentLength = image.size

            val tagImageFileName = generateFileName(prefix = "${tag.id}-")
            val tagImagePath = s3Uploader.upload(
                inputStream = image.inputStream,
                bucket = coverImageBucket,
                filePath = "live_cover/${tag.id}/$tagImageFileName",
                metadata = metadata
            )

            tag.image = tagImagePath
        }
    }

    @Transactional
    fun updateTagOrders(ids: List<Long>) {
        for (index in ids.indices) {
            val tag = repository.findByIdOrNull(ids[index])

            if (tag != null) {
                tag.orders = index + 1
            }
        }
    }

    fun getTags(member: Member): List<GetLiveTagResponse> {
        return repository.getTags(member = member, cloudFrontHost = cloudFrontHost)
    }

    fun tagExistCheck(request: CreateLiveTagRequest) {
        repository.findByTag(request.tag)?.let { throw SodaException("이미 등록된 태그 입니다.") }
    }
}
