package kr.co.vividnext.sodalive.member.auth

import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.bootpay.Bootpay
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.time.LocalDate

@Service
@Transactional(readOnly = true)
class AuthService(
    private val repository: AuthRepository,
    private val objectMapper: ObjectMapper,

    @Value("\${bootpay.application-id}")
    private val bootpayApplicationId: String,
    @Value("\${bootpay.private-key}")
    private val bootpayPrivateKey: String
) {
    @Transactional
    fun verify(member: Member, request: AuthVerifyRequest) {
        val bootpay = Bootpay(bootpayApplicationId, bootpayPrivateKey)
        try {
            val token = bootpay.accessToken
            if (token["error_code"] != null) throw SodaException("인증정보에 오류가 있습니다.\n다시 시도해 주세요.")

            val res = bootpay.certificate(request.receiptId)
            val certificateResult = objectMapper.convertValue(res, AuthCertificateResult::class.java)

            if (
                certificateResult.status == 12 &&
                certificateResult.statusLocale == "본인인증완료" &&
                certificateResult.receiptId == request.receiptId
            ) {
                val certificate = certificateResult.authenticateData
                val nowYear = LocalDate.now().year
                val certificateYear = certificate.birth.substring(0, 4).toInt()
                if (nowYear - certificateYear >= 19) {
                    val auth = Auth(
                        name = certificate.name,
                        birth = certificate.birth,
                        uniqueCi = certificate.unique,
                        di = certificate.di,
                        gender = certificate.gender
                    )
                    auth.member = member

                    repository.save(auth)
                } else {
                    throw SodaException("19세 미만 인증 오류")
                }
            } else {
                throw SodaException("인증정보에 오류가 있습니다.\n다시 시도해 주세요.")
            }
        } catch (e: Exception) {
            throw SodaException("인증정보에 오류가 있습니다.\n다시 시도해 주세요.")
        }
    }
}
